.include "m8def.inc"
.equ F_CPU = 16000000 ; Systemtakt in Hz


.CSEG
	rjmp RESET ; Reset Handler
	rjmp EXT_INT0 ; IRQ0 Handler
	reti ; rjmp EXT_INT1 ; IRQ1 Handler
	reti ; rjmp TIM2_COMP ; Timer2 Compare Handler
	reti ; rjmp TIM2_OVF ; Timer2 Overflow Handler
	rjmp TIM1_CAPT ; Timer1 Capture Handler
	reti ; rjmp TIM1_COMPA ; Timer1 CompareA Handler
	reti ; rjmp TIM1_COMPB ; Timer1 CompareB Handler
	rjmp TIM1_OVF ; Timer1 Overflow Handler
	reti ; rjmp TIM0_OVF ; Timer0 Overflow Handler
	reti ; rjmp SPI_STC ; SPI Transfer Complete Handler
	rjmp USART_RXC ; USART RX Complete Handler
	rjmp USART_UDRE ; UDR Empty Handler
	rjmp USART_TXC ; USART TX Complete Handler
	reti ; rjmp ADC_COMPLETE ; ADC Conversion Complete Handler
	reti ; rjmp EE_RDY ; EEPROM Ready Handler
	reti ; rjmp ANA_COMP ; Analog Comparator Handler
	reti ; rjmp TWSI ; Two-wire Serial Interface Handler
	reti ; rjmp SPM_RDY ; Store Program Memory Ready Handler


.equ SERIAL_BAUD = 9600
.include "serial.inc.asm"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Aufnahme des ein/aus-Verlaufs beim Prellen einer Taste und bertragung als 
;;  Ereignisprotokoll zur seriellen Schnittstelle.
;; Der 16-bit Timer wird mit presaler /1 (no prescaling; CS12=0, CS11=0, CS10=1) 
;;  betrieben, bei Overflow wird ein Zhler (bit 16-23 des Timestamps) 
;;  hochgezhlt. So knnen Zeitrume von etwa 1 Sekunde 
;;  (1,0485759375 = 0xffffff / 16.000.000) mit einer Auflsung von 
;;  62,5 ns / 0,0625 us / 0,0000625 ms beschrieben werden.
;; Es gibt 3 mgliche Varianten fr die Aufzeichnung.
;;  Welche der Optionen benutzt wird, wird ber die Definition von TASTENAUFNAHME_ALG
;;   bestimmt.
;;  1.: Taster an ICP1 (PB0) anschlieen
;;   .equ TASTENAUFNAHME_ALG = 1
;;   Bei jedem TIM1_CAPT-Ereignis den Timestamp in ICR1H:ICR1L in der Tabelle ablegen 
;;    und die Flankenrichtung wechseln, wie im Handbuch ATmega8 auf Seite 84 beschrieben:
;;    "Measurement of an external signals duty cycle requires that the trigger edge is changed after
;;     each capture. Changing the edge sensing must be done as early as possible after the ICR1
;;     Register has been read. After a change of the edge, the Input Capture Flag (ICF1) must be
;;     cleared by software (writing a logical one to the I/O bit location). For measuring frequency only,
;;     the clearing of the ICF1 Flag is not required (if an interrupt handler is used)."
;;   Da sich Ein- und Ausschaltvorgnge stets abwechseln, ist keine Speicherung
;;    des Zustandes ntig.
;;   Nachteil:
;;    Wenn einem Einschaltvorgang ein Auschaltvorgang folgt, bevor die Interrupt-
;;     Routine die Flankenrichtung gewechselt hat, wird dieser Ausschaltvorgang
;;     und der darauf folgende Einschaltvorgang verschluckt, und erst der wiederum
;;     darauf folgende Ausschaltvorgang registriert.
;;     Bei Tests war es in einem Fall so, dass der verschluckte Schaltvorgang
;;     der letzte beim Einschalten war, und so der Zeitraum, den die Taste gedrckt
;;     war, als ausgeschaltet protokolliert wurde.
;;  2.: Taster an INT0 (PD2) anschlieen
;;   .equ TASTENAUFNAHME_ALG = 2
;;   Hier wird durch jeden Ein- und Auschaltvorgang ein Interrupt ausgelst, ohne 
;;    dass die Flankenrichtung gewechselt werden muss. Wenn hier einem 
;;    Einschaltvorgang ein Auschaltvorgang folgt, bevor die Interrupt-
;;    Routine die Flankenrichtung gewechselt hat, wird zwar dieser Ausschaltvorgang
;;    nicht bemerkt, aber der darauf folgende Einschaltvorgang.
;;   Nachteile:
;;    - Der Zustand muss mit protokolliert werden, da z.B. zwei Einschaltvorgnge
;;    hintereinander folgen knnen.
;;    - Die Timestamps sind um ein paar Takte verschoben, da die Routine nicht auf
;;    ICR1H:ICR1L zurckgreifen kann, sondern den aktuellen Wert aus dem Timer-Registern
;;    lesen muss.
;;  3. Keine Interrupts bei Flankenwechsel, sondern eine Schleife, die ber den Messzeitraum
;;    einen Eingang abfragt, und nderungen protokolliert. Da der C sonst nichts zu tun hat,
;;    ist diese Option (trotz der tollen Interrupt-Mglichkeiten) plausibel.
;;   Als Input wird ebenfalls PD2 (INT0) verwendet.
;;   .equ TASTENAUFNAHME_ALG = 3
;;   
;; Speicherung von 1000 24-bit Timestamps in einer 3000 Byte groen Tabelle.
;; Steuerung ber RS-232:
;;  's': Starte Messung
;;  'a': Ausgabe im Format
;;        100000.0ns 0.0V
;;        100000.01ns 5.0V
;;        111234.5ns 5.0V
;;        111234.51ns 0.0V
;;
;; Die Umrechnung in die Ausgabeeinheit:
;; i = 24-Bit Timestamp (1/16000000 sek * i bzw. 0,0000625 ms * i) 
;; t = Zeit in ns (5 bytes bzw. 40 bit)
;;
;;  t = (i * 1000) / 16   | /8
;;  t = (i * 125) / 2      | /2
;;  t = (i * 62,5)
;; Multiplikation mit 125:
;;  i * 125 = (i * 128) - (i * 2) - i = (i << 7) - i - i - i
;; Wenn Ausgabeformat in 1/10 ns (x.y statt xy.0):
;; Multiplikation mit 625:
;;  i * 625 = (i*512) + (i*64) + (i*32) + (i*16) + i = (i << 9) + (i << 6) + (i << 5) + (i << 4) + i
;; 
;; Umrechnen in BCD-Format:
;; Die hchste 24-Bit Zahl ist 16777215, 
;; ins Ausgabeformat (t) umgerechnet 10485759375, also 11 Stellen.
;; Die Wortbreite ist also 32 bit bzw. 4 bytes.
;;
;; Ausgabe einer 11-stelligen BCD-Zahl im Ausgabeformat:
;;  1. Von der BCD-Zahl werden die ersten 9 Stellen ohne fhrende Nullen ausgegeben.
;;  2. Die vorletzte Ziffer wird ausgegeben
;;  3. ein '.' wird ausgegeben
;;  4. die letzte Ziffer wird ausgegeben.
;;  5. Der String "ns " wird ausgegeben
;;
;; Ausgabe eines Timestamps:
;;  LTSpice interpoliert Werte, was verhindert wird, indem jeder Timestamp als
;;  2 Zeilen mit unmittelbar folgenden Zeitpunkten ausgegeben wird, wobei
;;  die 1. Zeile den letzen Wert in der rechten Spalte angibt, und die 2. Zeile den
;;  neuen Wert.

.equ TASTENAUFNAHME_ALG = 3

; Konstanten:
Startmeldung:
	.db "Tastenprellen-Recorder",13,10," s: Start",13,10," a: Ausgeben",13,10,0

; Register:
; .def  = r0
.def rTstampL = r1
.def rTstampH = r2
.def rZeit0 = r3
.def rZeit1 = r4
.def rZeit2 = r5
.def rZeit3 = r6
.def rErg0 = r7
.def rErg1 = r8
.def rErg2 = r9
.def rErg3 = r10
.def rTabelleEndeL = r11
.def rTabelleEndeH = r12
; .def  = r13
; .def  = r14
; .def  = r15
.def rParameter = r16
.def rParameter2 = r17
.def rTmp = r18
.def rTmp2 = r19
.def rTmp3 = r20
.def rRest = r21
.if (TASTENAUFNAHME_ALG == 1)
	.def rTCCRB = r22
	.def rTCCRB_Mask = r23
.endif
.if ((TASTENAUFNAHME_ALG == 2) || (TASTENAUFNAHME_ALG == 3))
	.def rBitfeld = r22 ; Bit 0: letzter Wert; Bit 1: Aufhren
.endif
.def rZaehler = r24
.def rSpannung = r25

.equ BITFELD_LETZTER_WERT = 0
.equ BITFELD_BEENDEN = 1
.if (TASTENAUFNAHME_ALG == 3)
	.equ BITFELD_START_POLLING = 2
.endif

; RAM:
.DSEG
Tabelle:
	.byte(512)
TabelleEnde:
BCD_Erg:
	.byte(11)

.CSEG
RESET: ldi r16,high(RAMEND); Main program start
	out SPH,r16 ; Set Stack Pointer to top of RAM
	ldi r16,low(RAMEND)
	out SPL,r16
	sei ; Enable interrupts

	rcall SerialInit

	ldi zl,LOW(Startmeldung*2)
	ldi zh,HIGH(Startmeldung*2)
	rcall SerialStringAusgeben

.if (TASTENAUFNAHME_ALG == 1)
	; PB0 (ICP1) auf Input setzen und Pullup aktivieren
	cbi DDRB,0
	sbi PORTB,0
.endif
.if ((TASTENAUFNAHME_ALG == 2) || (TASTENAUFNAHME_ALG == 3))
	; PD2 (INT0) auf Input setzen und Pullup aktivieren
	cbi DDRD,2
	sbi PORTD,2
.endif

	
	ldi rTmp, LOW(TabelleEnde)
	mov rTabelleEndeL, rTmp
	ldi rTmp, HIGH(TabelleEnde)
	mov rTabelleEndeH, rTmp

	// Testing/Debugging:
	/*
	ldi rTmp,0x56
	mov rTstampL,rTmp
	ldi rTmp,0x34
	mov rTstampH,rTmp
	ldi rTmp,0x12
	mov rZaehler,rTmp
	rcall Multipliziere62Komma5_24bitnach32bit
	; Parameter:
	;  rTstampL, rTstampH, rZaehler
	; Ergebnis:
	;  rZeit0, rZeit1, rZeit2, rZeit3
	*/
	; Multipliziere62Komma5_24bitnach32bit testen! => OK
	; Automatisches Beenden der Messung bei berlauf implementieren! => OK

MAINLOOP:
.if (TASTENAUFNAHME_ALG == 3)
	sbrc rBitfeld, BITFELD_START_POLLING
	rcall AufnahmePolling
.endif
	rjmp MAINLOOP

SerialZeichenVerarbeiten:
	cpi r16,'s'
	breq SerialZeichenVerarbeiten_s
	cpi r16,'a'
	breq SerialZeichenVerarbeiten_a
	ret
SerialZeichenVerarbeiten_s:
	rcall StarteAufnahme
	ret
SerialZeichenVerarbeiten_a:
	; Iteriere durch die Tabelle und gebe die Timestamps und Spannungen aus.
	ldi   ZL,LOW(Tabelle)
	ldi   ZH,HIGH(Tabelle)
	clr   rSpannung
	; com   rSpannung
SchleifeSerialZeichenVerarbeiten_a:
	cp    ZL, XL
	cpc   ZH, XH
	breq  EndeSerialZeichenVerarbeiten_a
	ld    rTstampL, Z+
	ld    rTstampH, Z+
	ld    rZaehler, Z+
	rcall TimestampAusgeben
	ldi   rParameter, '0'
	rcall SerialTXCharNow
	rcall nsPostfixAusgeben
	rcall SpannungAusgeben
	com   rSpannung
	rcall TimestampAusgeben
	ldi   rParameter, '1'
	rcall SerialTXCharNow
	rcall nsPostfixAusgeben
	rcall SpannungAusgeben
	rjmp  SchleifeSerialZeichenVerarbeiten_a
EndeSerialZeichenVerarbeiten_a:
	ret

snsPostfix:
	.db "ns ",0

nsPostfixAusgeben:
	push  ZL
	push  ZH
	ldi   ZL,LOW(snsPostfix*2)
	ldi   ZH,HIGH(snsPostfix*2)
	rcall SerialStringAusgeben
	pop   ZH
	pop   ZL
	ret

sSpannungHigh:
	.db "5.0V",13,10,0,0
sSpannungLow:
	.db "0.0V",13,10,0,0

SpannungAusgeben:
	push  ZL
	push  ZH
	cpi   rSpannung, 0
	brne  SpannungAusgebenHigh
	ldi   ZL,LOW(sSpannungLow*2)
	ldi   ZH,HIGH(sSpannungLow*2)
	rcall SerialStringAusgeben
	rjmp SpannungAusgebenEnde
SpannungAusgebenHigh:
	ldi   ZL,LOW(sSpannungHigh*2)
	ldi   ZH,HIGH(sSpannungHigh*2)
	rcall SerialStringAusgeben
SpannungAusgebenEnde:
	pop   ZH
	pop   ZL
	ret

TimestampAusgeben:
	; Parameter:
	;  rTstampL, rTstampH, rZaehler
	rcall Multipliziere62Komma5_24bitnach32bit
	ldi   YL,LOW(BCD_Erg)
	ldi   YH,HIGH(BCD_Erg)
	rcall Binaer32NachBCD
	; 9 Stellen ausgeben
	; rTmp: Down-Zhler fr Stellen
	; rTmp2: 0-Flag (0: fhrende Nullen aktiv, 1: in der Zahl)
	; rParameter: Wert
	ldi   rTmp, 9
	ldi   rTmp2, 0
SchleifeTimestampAusgeben:
	ld    rParameter, Y+
	cpi   rParameter, 0
	brne  AusgabeTimestampAusgeben
	cpi   rTmp2, 0
	brne  AusgabeTimestampAusgeben
	rjmp  WeiterTimestampAusgeben
AusgabeTimestampAusgeben:
	ldi   rTmp2, 1
	subi  rParameter, -'0'
	rcall SerialTXCharNow
WeiterTimestampAusgeben:
	dec   rTmp
	brne  SchleifeTimestampAusgeben
	ld    rParameter, Y+
	subi  rParameter, -'0'
	rcall SerialTXCharNow
	ldi   rParameter, '.'
	rcall SerialTXCharNow
	; ld    rParameter, Y+
	; subi  rParameter, -'0'
	; rcall SerialTXCharNow
	ret

StarteAufnahme:
	; Zeiger initialisieren:
	ldi XL,LOW(Tabelle)
	ldi XH,HIGH(Tabelle)
	; Zhler initialisieren:
	ldi rZaehler,0
	; Timer und Maske initialisieren
	ldi rTmp, 0
	out ICR1H, rTmp
	out ICR1L, rTmp
	out TCCR1A, rTmp ; Kein Compare Output Modus, kein Waveform Generation Modus, 
.if (TASTENAUFNAHME_ALG == 1)
	ldi rTmp, (1 << TICIE1) | (1 << TOIE1); Interrupts bei Input Capture und Overflow
	ldi rTCCRB_Mask, 1 << ICES1
	ldi rTmp2, (1 << CS10) ; fallende Flanke, kein Prescaling
	ldi rTmp3, 0 ; Keine externen Interrupts
.endif
.if (TASTENAUFNAHME_ALG == 2)
	ldi rBitfeld, BITFELD_LETZTER_WERT
	ldi rTmp, (1 << TOIE1); Interrupt bei Overflow
	ldi rTmp2, (1 << CS10) ; kein Prescaling
	ldi rTmp3, (1 << INT0)
	out GICR, rTmp3
	ldi rTmp3, (1 << ISC00) ; Any logical change on INT0 generates an interrupt request.
.endif
.if (TASTENAUFNAHME_ALG == 3)
	ldi rBitfeld, (1 << BITFELD_START_POLLING)
	ldi rTmp, (1 << TOIE1); Interrupt bei Overflow
	ldi rTmp2, (1 << CS10) ; kein Prescaling
	ldi rTmp3, 0 ; Keine externen Interrupts
.endif
	out MCUCR, rTmp3
	out TCCR1B, rTmp2
	out TIMSK, rTmp
	ret

.if (TASTENAUFNAHME_ALG == 3)
AufnahmePolling:
; Warte auf Low Potential am Pin (Taster ein) oder Ende-Flag
AufnahmePollingSchleifeAussen:
AufnahmePollingSchleife1:
	sbrc rBitfeld, BITFELD_BEENDEN
	rjmp AufnahmePollingEnde
	sbic PIND, 2
	rjmp AufnahmePollingSchleife1
	in   rTstampL, TCNT1L
	in   rTstampH, TCNT1H
	st   X+, rTstampL
	st   X+, rTstampH
	st   X+, rZaehler
	cp   XL, rTabelleEndeL
	cpc  XH, rTabelleEndeH
	brsh AufnahmePollingEnde
; Warte auf High Potential am Pin (Taster ein) oder Ende-Flag
AufnahmePollingSchleife2:
	sbrc rBitfeld, BITFELD_BEENDEN
	rjmp AufnahmePollingEnde
	sbis PIND, 2
	rjmp AufnahmePollingSchleife2
	in   rTstampL, TCNT1L
	in   rTstampH, TCNT1H
	st   X+, rTstampL
	st   X+, rTstampH
	st   X+, rZaehler
	cp   XL, rTabelleEndeL
	cpc  XH, rTabelleEndeH
	brsh AufnahmePollingEnde
	rjmp AufnahmePollingSchleifeAussen
AufnahmePollingEnde:
	cbr rBitfeld, BITFELD_START_POLLING
	ret
.endif 

BeendeAufnahme:
	sbr rBitfeld, BITFELD_BEENDEN
	push rTmp
	ldi rTmp, 0
	out TIMSK, rTmp
	out TCCR1B, rTmp
	pop rTmp
	ret

Multipliziere625_24bitnach32bit:
	; Parameter:
	;  rTstampL, rTstampH, rZaehler
	; Ergebnis:
	;  rZeit0, rZeit1, rZeit2, rZeit3
	; Mit 512 multiplizieren:

Multipliziere62Komma5_24bitnach32bit:
	; Parameter:
	;  rTstampL, rTstampH, rZaehler
	; Ergebnis:
	;  rZeit0, rZeit1, rZeit2, rZeit3
	ldi rTmp, 0
	; Mit 128 multiplizieren:
	mov rZeit3, rZaehler
	lsr rZeit3
	mov rZeit2, rTstampH
	ror rZeit2
	mov rZeit1, rTstampL
	ror rZeit1
	clr rZeit0
	ror rZeit0
	; 3 * Paramter abziehen. Macht aus der Multiplikation mit 128 eine 
	;  Multiplikation mit 125 (128 * i - 3 * i = 125 * i)
	ldi rTmp2, 3
SchliefeMultipliziere62Komma5_24bitnach32bit:
	sub rZeit0, rTstampL
	sbc rZeit1, rTstampH
	sbc rZeit2, rZaehler
	sbc rZeit3, rTmp
	dec rTmp2
	brne SchliefeMultipliziere62Komma5_24bitnach32bit
	; Das Zwischenergebnis / 2 teilen:
	lsr rZeit3
	ror rZeit2
	ror rZeit1
	ror rZeit0
	ret

Binaer32NachBCD:
	; Parameter;
	;  rZeit0, rZeit1, rZeit2, rZeit3
	; Ergebnis: 
	;  [Y-Zeiger] (10 Stellen mit fhrenden Nullen)
	push  rTmp
	push  rTmp2
	ldi   rTmp,10
	clr   rTmp2
	add   YL, rTmp
	adc   YH, rTmp2
SchleifeBinaer32NachBCD:
	rcall TeileDurch10_32bit
	st    -Y, rRest
	mov   rZeit0, rErg0
	mov   rZeit1, rErg1
	mov   rZeit2, rErg2
	mov   rZeit3, rErg3
	dec   rTmp
	brne  SchleifeBinaer32NachBCD
	pop   rTmp2
	pop   rTmp
	ret

TeileDurch10_32bit:
	; Parameter:
	;  rZeit0, rZeit1, rZeit2, rZeit3
	; Ergebnis:
	;  rErg0, rErg1, rErg2, rErg3; Rest rRest
	push rTmp2
	push rTmp3
	push rZeit0
	push rZeit1
	push rZeit2
	push rZeit3
	clr rErg0
	clr rErg1
	clr rErg2
	clr rErg3
	clr rRest
	clt
	ldi rTmp2, 9
	ldi rTmp3, 8
SchleifeTeileDurch10_32bit3:
	lsl rZeit3 ; linkestes Bit in Carry-Flag
	rol rRest ; Dieses als rechtestes Bit in den Zwischendividend
	cp rTmp2, rRest ; Wenn rTmp > 9, setze Carry-Flag 
	rol rErg3 ; hole dieses Carry-Flag in das Ergebnis
	sbrc rErg3,0 ; Das rechtste Bit vom Ergebnis
	subi rRest, 10 ; mit 10 multipliziert vom Zwischendividend subtrahieren
	dec rTmp3
	brne SchleifeTeileDurch10_32bit3
	ldi rTmp3, 8
SchleifeTeileDurch10_32bit2:
	lsl rZeit2 ; linkestes Bit in Carry-Flag
	rol rRest ; Dieses als rechtestes Bit in den Zwischendividend
	cp rTmp2, rRest ; Wenn rTmp > 9, setze Carry-Flag 
	rol rErg2 ; hole dieses Carry-Flag in das Ergebnis
	sbrc rErg2,0 ; Das rechtste Bit vom Ergebnis
	subi rRest, 10 ; mit 10 multipliziert vom Zwischendividend subtrahieren
	dec rTmp3
	brne SchleifeTeileDurch10_32bit2
	ldi rTmp3, 8
SchleifeTeileDurch10_32bit1:
	lsl rZeit1 ; linkestes Bit in Carry-Flag
	rol rRest ; Dieses als rechtestes Bit in den Zwischendividend
	cp rTmp2, rRest ; Wenn rTmp > 9, setze Carry-Flag 
	rol rErg1 ; hole dieses Carry-Flag in das Ergebnis
	sbrc rErg1,0 ; Das rechtste Bit vom Ergebnis
	subi rRest, 10 ; mit 10 multipliziert vom Zwischendividend subtrahieren
	dec rTmp3
	brne SchleifeTeileDurch10_32bit1
	ldi rTmp3, 8
SchleifeTeileDurch10_32bit0:
	lsl rZeit0 ; linkestes Bit in Carry-Flag
	rol rRest ; Dieses als rechtestes Bit in den Zwischendividend
	cp rTmp2, rRest ; Wenn rTmp > 9, setze Carry-Flag 
	rol rErg0 ; hole dieses Carry-Flag in das Ergebnis
	sbrc rErg0,0 ; Das rechtste Bit vom Ergebnis
	subi rRest, 10 ; mit 10 multipliziert vom Zwischendividend subtrahieren
	dec rTmp3
	brne SchleifeTeileDurch10_32bit0
	pop rZeit3
	pop rZeit2
	pop rZeit1
	pop rZeit0
	pop rTmp3
	pop rTmp2
	ret
	
TIM1_CAPT: ; Timer1 Capture Handler
.if (TASTENAUFNAHME_ALG == 1)
	in   rTstampL, ICR1L
	in   rTstampH, ICR1H
	in   rTCCRB, TCCR1B
	eor  rTCCRB, rTCCRB_Mask ; invertiere ICES1
	out  TCCR1B, rTCCRB
	sbi  TIFR ,ICF1 ; Siehe oben. (After a change of the edge, the Input Capture Flag (ICF1) must..)
	st   X+, rTstampL
	st   X+, rTstampH
	st   X+, rZaehler
	cp   XL, rTabelleEndeL
	cpc  XH, rTabelleEndeH
	brsh TIM1_CAPT_BeendeAufnahme
	reti
TIM1_CAPT_BeendeAufnahme:
	rcall BeendeAufnahme
	push rParameter
	ldi rParameter,'b'
	rcall SerialTXCharNow
	pop rParameter
.endif
	reti

TIM1_OVF: ; Timer1 Overflow Handler
	inc   rZaehler
	breq  TIM1_OVF_BeendeAufnahme
	reti
TIM1_OVF_BeendeAufnahme:
	rcall BeendeAufnahme
	push rParameter
	ldi rParameter,'B'
	rcall SerialTXCharNow
	pop rParameter
	reti

EXT_INT0: ; IRQ0 Handler
.if (TASTENAUFNAHME_ALG == 2)
	in rTstampL, TCNT1L
	in rTstampH, TCNT1H
	sbrs rBitfeld, BITFELD_LETZTER_WERT
	rjmp EXT_INT0_LetzterWert0
EXT_INT0_LetzterWert1: ; Kommentar-Label
	sbic PIND, 2
	reti
	cbr  rBitfeld, BITFELD_LETZTER_WERT
	st   X+, rTstampL
	st   X+, rTstampH
	st   X+, rZaehler
	cp   XL, rTabelleEndeL
	cpc  XH, rTabelleEndeH
	brsh EXT_INT0_BeendeAufnahme
	reti
EXT_INT0_LetzterWert0:
	sbis PIND, 2
	reti
	sbr  rBitfeld, BITFELD_LETZTER_WERT
	st   X+, rTstampL
	st   X+, rTstampH
	st   X+, rZaehler
	cp   XL, rTabelleEndeL
	cpc  XH, rTabelleEndeH
	brsh EXT_INT0_BeendeAufnahme
	reti
EXT_INT0_BeendeAufnahme:
	rcall BeendeAufnahme
	push rParameter
	ldi rParameter,'i'
	rcall SerialTXCharNow
	pop rParameter
.endif
	reti

/*
EXT_INT1: ; IRQ1 Handler
	reti
TIM2_COMP: ; Timer2 Compare Handler
	reti
TIM2_OVF: ; Timer2 Overflow Handler
	reti
TIM1_COMPA: ; Timer1 CompareA Handler
	reti
TIM1_COMPB: ; Timer1 CompareB Handler
	reti
TIM0_OVF: ; Timer0 Overflow Handler
	reti
SPI_STC: ; SPI Transfer Complete Handler
	reti
ADC_COMPLETE: ; ADC Conversion Complete Handler
	ret
EE_RDY: ; EEPROM Ready Handler
	reti
ANA_COMP: ; Analog Comparator Handler
	reti
TWSI: ; Two-wire Serial Interface Handler
	reti
SPM_RDY: ; Store Program Memory Ready Handler
	reti
*/
